==> chess_logic-1.0.0.tar.gz: lib/src/chess_board.dart <==
import 'package:utilary/utilary.dart' show Vector, Rectangle, Direction;
import 'chessman.dart';

class ChessBoard {
  static const ranks = 8;
  static const files = 8;

  static const List<ChessColor> _players = [ChessColor.white, ChessColor.black];

  final Rectangle _bounds = Rectangle(0, 0, 8, 8);
  Rectangle get bounds => _bounds;

  final Map<Vector, Chessman> _board;

  final List<Chessman> _pieces;
  List<Chessman> get pieces => _pieces;

  void add(Chessman piece) {
    _pieces.add(piece);
    _board[piece.pos] = piece;
  }

  void remove(Chessman piece) {
    _pieces.remove(piece);
    _board[piece.pos] = null;
  }

  void move(Chessman piece, Vector destination) {
    Chessman piaceAtDestination = _board[destination];
    //Check for capture
    if (piaceAtDestination != null) remove(piaceAtDestination);

    _board[piece.pos] = null;
    _board[destination] = piece;

    piece.move(destination);
  }

  ///Wheter any piece of [attacker] can move to [pos]
  bool canBeAttacked(Vector pos, ChessColor attacker) =>
      getPieces(attacker).any((Chessman piece) => piece.canMoveTo(pos));

  ///Returns `true` when these conditions are verified simultaniously:
  ///+ King of [color] is checked (in danger);
  ///+ The king is unable to move;
  ///+ The attack can't be neutralized: either by blocking it with another chessman
  ///  or capturing the attacker.
  bool isCheckMate(ChessColor color) {
    final Chessman king = getKing(color);

    //The king isn't even in danger..
    if (!king.isInDanger) return false;

    //If the king can move out of check there's no mate
    if (Direction.all
        .map((Direction dir) => dir + king.pos)
        .where((Vector pos) => bounds.contains(pos))
        .any((Vector pos) => !canBeAttacked(pos, complement(color)))) {
      return false;
    }

    //All the enemies threatening the king
    final List<Chessman> attackers = getPieces(complement(color))
        .where((Chessman piece) => piece.canMoveTo(king.pos));

    //Check whether the attack can be neutralized
    ///TODO: add other ways to neutralize an attack
    if (attackers.length == 1 &&
        getPieces(color)
            .any((Chessman piece) => piece.canMoveTo(attackers.first.pos))) {
      return false;
    }

    return true;
  }

  List<Chessman> getPieces(ChessColor color) =>
      color == ChessColor.white ? whites : blacks;

  ChessColor complement(ChessColor color) =>
      color == ChessColor.white ? ChessColor.black : ChessColor.white;

  List<Chessman> get whites => getPieces(ChessColor.white);
  List<Chessman> get blacks => getPieces(ChessColor.black);
  List<Chessman> get playerPieces => getPieces(turn);
  List<Chessman> get enemyPieces => getPieces(complement(turn));

  King get whiteKing => whites.whereType<King>().first;
  King get blackKing => blacks.whereType<King>().first;
  King getKing(ChessColor color) =>
      color == ChessColor.white ? whiteKing : blackKing;
  King get king => getKing(turn);
  King get enemyKing => getKing(complement(turn));

  ChessBoard.emptyBoard()
      : _board = Map(),
        _pieces = List();

  void operator []=(Vector pos, Chessman piece) => _board[pos] = piece;
  Chessman operator [](Vector pos) => _board[pos];

  Chessman get(int file, int rank) => _board[Vector(file, rank)];

  //White always start first
  int _turnIndex = 0;
  ChessColor get turn => _players[_turnIndex];
  ChessColor get enemy => complement(turn);
  void nextTurn() => _turnIndex = (_turnIndex + 1) % _players.length;
}

==> chess_logic-1.0.1.tar.gz: lib/src/chess_board.dart <==
import 'package:utilary/utilary.dart' show Vector, Rectangle, Direction;
import 'chessman.dart';

class ChessBoard {
  static const ranks = 8;
  static const files = 8;

  static const List<ChessColor> _players = [ChessColor.white, ChessColor.black];

  final Rectangle _bounds = Rectangle(0, 0, 8, 8);
  Rectangle get bounds => _bounds;

  final Map<Vector, Chessman> _board;

  final List<Chessman> _pieces;
  List<Chessman> get pieces => _pieces;

  void add(Chessman piece) {
    _pieces.add(piece);
    _board[piece.pos] = piece;
  }

  void remove(Chessman piece) {
    _pieces.remove(piece);
    _board[piece.pos] = null;
  }

  void move(Chessman piece, Vector destination) {
    Chessman piaceAtDestination = _board[destination];
    //Check for capture
    if (piaceAtDestination != null) remove(piaceAtDestination);

    _board[piece.pos] = null;
    _board[destination] = piece;

    piece.move(destination);
  }

  ///Wheter any piece of [attacker] can move to [pos]
  bool canBeAttacked(Vector pos, ChessColor attacker) =>
      getPieces(attacker).any((Chessman piece) => piece.canMoveTo(pos));

  ///Returns `true` when these conditions are verified simultaniously:
  ///+ King of [color] is checked (in danger);
  ///+ The king is unable to move;
  ///+ The attack can't be neutralized: either by blocking it with another chessman
  ///  or capturing the attacker.
  bool isCheckMate(ChessColor color) {
    final Chessman king = getKing(color);

    //The king isn't even in danger..
    if (!king.isInDanger) return false;

    //If the king can move out of check there's no mate
    if (Direction.all
        .map((Direction dir) => dir + king.pos)
        .where((Vector pos) => bounds.contains(pos))
        .any((Vector pos) => !canBeAttacked(pos, complement(color)))) {
      return false;
    }

    //All the enemies threatening the king
    final List<Chessman> attackers = getPieces(complement(color))
        .where((Chessman piece) => piece.canMoveTo(king.pos));

    //Check whether the attack can be neutralized
    ///TODO: add other ways to neutralize an attack
    if (attackers.length == 1 &&
        getPieces(color)
            .any((Chessman piece) => piece.canMoveTo(attackers.first.pos))) {
      return false;
    }

    return true;
  }

  List<Chessman> getPieces(ChessColor color) =>
      color == ChessColor.white ? whites : blacks;

  ChessColor complement(ChessColor color) =>
      color == ChessColor.white ? ChessColor.black : ChessColor.white;

  List<Chessman> get whites => pieces.where((Chessman piece) => piece.color == ChessColor.white).toList();
  List<Chessman> get blacks => pieces.where((Chessman piece) => piece.color == ChessColor.black).toList();
  List<Chessman> get playerPieces => getPieces(turn);
  List<Chessman> get enemyPieces => getPieces(complement(turn));

  King get whiteKing => whites.whereType<King>().first;
  King get blackKing => blacks.whereType<King>().first;
  King getKing(ChessColor color) =>
      color == ChessColor.white ? whiteKing : blackKing;
  King get king => getKing(turn);
  King get enemyKing => getKing(complement(turn));

  ChessBoard.emptyBoard()
      : _board = Map(),
        _pieces = List();

  void operator []=(Vector pos, Chessman piece) => _board[pos] = piece;
  Chessman operator [](Vector pos) => _board[pos];

  Chessman get(int file, int rank) => _board[Vector(file, rank)];

  //White always start first
  int _turnIndex = 0;
  ChessColor get turn => _players[_turnIndex];
  ChessColor get enemy => complement(turn);
  void nextTurn() => _turnIndex = (_turnIndex + 1) % _players.length;
}
